@@ -5,7 +5,21 @@ forestFloor = function(rf.fit,
                        binary_reg = FALSE,
                        bootstrapFC = FALSE,
                        ...) {
-  Class = class(rf.fit)[1] #read only first class
+  
+  #A very loose initial class check to avoid non-informatives error messsages
+  if(!any(class(rf.fit) %in% c("randomForest","train","forestFloor_external","list"))) {
+    stop("The rf.fit argument is not valid, should be 'randomForest' or 'train'")
+  }
+
+  #extract random model from caret object
+  if(inherits(rf.fit,"train")) {
+    message("This seems to be an caret object. Trying to extract 'randomForest' model")
+    message("caret support is experimental, see help(forestFloor) for an example with caret")
+    message("ask/complain freely at https://github.com/sorhawell/forestFloor/issues/27")
+
+    #simple as that...
+    rf.fit = rf.fit$finalModel
+  }
   
   #convert randomForest.formula
   if(inherits(rf.fit,"randomForest.formula")){
@@ -13,23 +27,39 @@ forestFloor = function(rf.fit,
     rn <- row.names(X)
     Terms <- delete.response(rf.fit$terms)
     X <- model.frame(Terms, X, na.action = na.fail)
+    if(!is.null(Xtest)){
+      Xtest <- model.frame(Terms, Xtest, na.action = na.fail)
+    }
   }
-    
+  
+  #warn and someday fix not-inbag/not-OOB problems
+  if(any(is.na(rf.fit$predicted))) {
+    warning("forestFloor: NA predictions in rf-object. Try to train with more trees(ntree)")
+    message("forestFloor: NA predictions in rf-object. Try to train with more trees(ntree)")
+    # is.na.ind = which(is.na(rf.fit$predicted))
+    # rf$predicted = rf$predicted[-is.na.ind ]
+    # rf$oob.times = rf$oob.times[-is.na.ind ]
+    # rf$votes     = rf$votes    [-is.na.ind,]
+    # X = X[-is.na.ind,]
+    # #length(rf$y)
+    # #sapply(rf,length)
+  }
+  
   #randomForest::randomForest or trimTrees::cinbag or rfPermute::rfPermute
   if(inherits(rf.fit,"randomForest")) {
-    if(Class=="rfPermute") print("class 'rfPermute' supported as 'randomForest'")
+    if(inherits(rf.fit,"rfPermute")) print("class 'rfPermute' supported as 'randomForest'")
     Type = rf.fit$type
     #changed classification to binary regression if requested and only two classes
     if(binary_reg) {
       if(!is.null(rf.fit$forest$nclass) && rf.fit$forest$nclass==2) {
         Type="regression"
       } else {
-        warning("binary_reg=T is not possible for >2 classes. 
+        warning("binary_reg=T is not possible for >2 classes.
                 Continue computation as multiClass")
-        
+
       }
     }
-  
+
     #dispatch either forestFloor_regression(and binary) or multiClassification
     switch(Type,
            regression = return(
@@ -54,15 +84,14 @@ forestFloor = function(rf.fit,
            ),#majorityTerminal
            stop("type of randomForest object is neither 'regression' or 'classification', (RF.fit$type==?)"))
   }
-  
+
   #other classes not supported...
-  if(Class=="forestFloor_external") {
+  if(inherits(rf.fit,"forestFloor_external")) {
     print("forestFloor_external is a standardised treemodelfit which is not implemented yet")
     return("... cold emptyness")
   }
-  
+
   #if not classses recognized
   stop("This class is not yet supported, is this a random forest model fit?")
 
 }
-   
\ No newline at end of file
